@@ -13,21 +13,26 @@ import org.gcube.portlets.user.tdtemplate.client.event.TemplateComplitedEvent;
 import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
 import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDefinition;
 import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDialogManager;
+import org.gcube.portlets.user.tdtemplate.client.template.view.suggestion.ConstraintSuggestionLabel;
+import org.gcube.portlets.user.tdtemplate.client.template.view.suggestion.SuggestionContainer;
 
 import com.extjs.gxt.ui.client.Style.ButtonScale;
 import com.extjs.gxt.ui.client.Style.IconAlign;
+import com.extjs.gxt.ui.client.Style.LayoutRegion;
 import com.extjs.gxt.ui.client.Style.Scroll;
 import com.extjs.gxt.ui.client.event.BaseEvent;
 import com.extjs.gxt.ui.client.event.ButtonEvent;
 import com.extjs.gxt.ui.client.event.Events;
 import com.extjs.gxt.ui.client.event.Listener;
 import com.extjs.gxt.ui.client.event.SelectionListener;
+import com.extjs.gxt.ui.client.util.Margins;
 import com.extjs.gxt.ui.client.widget.ContentPanel;
 import com.extjs.gxt.ui.client.widget.LayoutContainer;
 import com.extjs.gxt.ui.client.widget.button.Button;
-import com.extjs.gxt.ui.client.widget.button.ButtonGroup;
-import com.extjs.gxt.ui.client.widget.form.CheckBox;
+import com.extjs.gxt.ui.client.widget.layout.BorderLayout;
+import com.extjs.gxt.ui.client.widget.layout.BorderLayoutData;
 import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
+import com.extjs.gxt.ui.client.widget.toolbar.FillToolItem;
 import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
 import com.google.gwt.core.client.GWT;
 import com.google.gwt.user.client.ui.AbstractImagePrototype;
@@ -45,25 +50,25 @@ import com.google.gwt.user.client.ui.Widget;
  */
 public class TemplatePanel {
 	
-
-	/**
-	 * 
-	 */
-	
-	/**
-	 * An instance of the constants.
-	 */
 	protected FlexTable flexTableTemplate = new FlexTable();
+	protected LayoutContainer flexTableContainer = new LayoutContainer();
 	private int numColumns;
 	protected ContentPanel tableContainer = new ContentPanel();
-	private ContentPanel suggestionContainer = new ContentPanel();
-	private LayoutContainer suggestionDescription = new LayoutContainer();
-	private LayoutContainer principalContainer = new LayoutContainer();
+	private SuggestionContainer suggestionContainer = new SuggestionContainer();
 	
 	protected List<ColumnDefinitionView> columnsDefined;
 	private TemplateSwitcherInteface templateSwitcherInteface;
 	protected SetColumnTypeDefinition setColumnTypeDefinition;
-	private TdTemplateController controller;
+	protected TdTemplateController controller;
+	
+	private LayoutContainer centralContainer = new LayoutContainer();
+	
+	protected ContentPanel principalContainer;
+	protected ContentPanel southContainer;
+//	protected Button filterBySelection;
+	private BorderLayout borderLayout;
+	private BorderLayoutData centerData;
+	private BorderLayoutData southData;
 	
 	
 	/**
@@ -98,32 +103,26 @@ public class TemplatePanel {
 		this.tableContainer.setScrollMode(Scroll.AUTOX);
 		this.tableContainer.setEnabled(false);
 		
-//		suggestionContainer.setStyleAttribute("margin-top", "5px");
-//		suggestionContainer.setStyleAttribute("margin-bottom", "10px");
-		suggestionContainer.setScrollMode(Scroll.AUTOY);
-		suggestionContainer.setHeaderVisible(false);
-		suggestionContainer.setBorders(false);
-		suggestionContainer.setBodyBorder(true);
-		
-		
-		suggestionContainer.add(suggestionDescription);
-
-//		suggestionContainer.add(constraintSuggestion.getHtml());
-		
-		addNewSuggestion(TdTemplateConstants.SUGGESTION, "Please, set type to all columns");
+		refreshSuggestion(TdTemplateConstants.SUGGESTION, "Please, set type to all columns");
 //		suggestionContainer.getElement().getStyle().setBorderColor("#32CD32");
 
+		tableContainer.setBodyBorder(false);
+		tableContainer.setBorders(false);
+		
 		tableContainer.add(suggestionContainer);
-		tableContainer.add(flexTableTemplate);
+		
+		flexTableContainer.add(flexTableTemplate);
+		tableContainer.add(flexTableContainer);
 		
 		
 		String title = templateSwitcherInteface.getType() + " columns constraints";
 		ConstraintSuggestionLabel cs = new ConstraintSuggestionLabel(title, templateSwitcherInteface.getTdTTemplateType().getConstraintDescription(), false);
 //		tableContainer.add(cs.getAnchor());
 
-		principalContainer.add(tableContainer);
-		principalContainer.add(cs);
-
+		centralContainer.add(tableContainer);
+		centralContainer.setBorders(false);
+		centralContainer.add(cs);
+		centralContainer.setScrollMode(Scroll.AUTOY);
 
 		suggestionContainer.addListener(Events.Render, new Listener<BaseEvent>() {
 
@@ -134,11 +133,52 @@ public class TemplatePanel {
 			}
 		});
 		
+		createBoderLayout();
+//		enableFilter(false);
 	}
 	
 
+	/**
+	 * 
+	 */
+	private void createBoderLayout() {
+		
+		borderLayout = new BorderLayout();
+		principalContainer = new ContentPanel(borderLayout);
+		principalContainer.setHeaderVisible(false);
+
+		centerData = new BorderLayoutData(LayoutRegion.CENTER);  
+		centerData.setMargins(new Margins(0));		
+		
+	    southData = new BorderLayoutData(LayoutRegion.SOUTH, 200);  
+	    southData.setSplit(true);  
+	    southData.setCollapsible(true);  
+	    southData.setFloatable(true);  
+	    southData.setMargins(new Margins(5,0,0,0));  
+//	    southData.setMaxSize(250);
+//	    southData.setMinSize(50);
+		southContainer = new ContentPanel();
+		southContainer.setHeaderVisible(true);
+		southContainer.setBorders(false);
+		southContainer.setBodyBorder(false);
+		southContainer.setHeading("Filters");
+		southContainer.setScrollMode(Scroll.AUTOY);
+		
+		principalContainer.add(centralContainer, centerData);
+//		principalContainer.add(southContainer, southData);
+		
+	}
 	
+	public void refreshSuggestion(String title, String text){
+		suggestionContainer.setSuggestion(title, text);
+		tableContainer.layout();
+	}
 	
+	public void refreshSuggestion(String title, String text, AbstractImagePrototype img){
+		suggestionContainer.setSuggestion(title, text,  "", img);
+		tableContainer.layout();
+	}
+
 	public void setBorderAsOnError(boolean bool){
 		
 		if(suggestionContainer.isRendered() && suggestionContainer.getElement("body")!=null){
@@ -149,23 +189,23 @@ public class TemplatePanel {
 		}
 	}
 	
-	public void addNewSuggestion(String title, String text){
-		SuggestionLabel sugg = new SuggestionLabel(title, text, "");
-		addSuggestionLabel(sugg);
+//	private void setBodyColumnOpacity(FlexTable table, int colIndex, double opacity) {
+//		for (int i = 1; i < table.getRowCount(); i++){
+////			table.getCellFormatter().setVisible(i, colIndex, b);
+//			table.getCellFormatter().getElement(i, colIndex).getStyle().setOpacity(opacity);
+//		}
+//	}
+//	
+	private void setColumnTypeOpacity(int row, int colIndex, double opacity) {
+		flexTableTemplate.getCellFormatter().getElement(row, colIndex).getStyle().setOpacity(opacity);
 	}
 	
-	public void addNewSuggestion(String title, String text, AbstractImagePrototype img){
-		SuggestionLabel sugg = new SuggestionLabel(title, text,  "", img);
-		addSuggestionLabel(sugg);
-	}
-	
-	private void addSuggestionLabel(SuggestionLabel sugg){
-		suggestionDescription.removeAll();
-		suggestionDescription.add(sugg.getHtml());
-		tableContainer.layout();
+	private void setColumnTypeOpacity(int colIndex, double opacity) {
+		setColumnTypeOpacity(1, colIndex, opacity);
 	}
 
 
+	
 	/**
 	 * 
 	 */
@@ -173,8 +213,8 @@ public class TemplatePanel {
 		
 		ToolBar toolbar = new ToolBar();
 		
-		ButtonGroup group = new ButtonGroup(1);  
-	    group.setHeading("Modify Column");  
+//		ButtonGroup group = new ButtonGroup(1);  
+//	    group.setHeading("Modify Column");  
 
 
 		// Add a button that will add more rows to the table
@@ -209,14 +249,94 @@ public class TemplatePanel {
 			}
 		});
 		
+		/*
+		filterBySelection = new Button("Filter by Selection", TdTemplateAbstractResources.filter16());
+		filterBySelection.setScale(ButtonScale.SMALL);
+		filterBySelection.setIconAlign(IconAlign.LEFT);
+		filterBySelection.setToolTip("Filter by Check Column");
+//		removeRowButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
+		
+		filterBySelection.addSelectionListener(new SelectionListener<ButtonEvent>() {
+
+			@Override
+			public void componentSelected(ButtonEvent ce) {
+
+				List<ColumnElement> columnsSelected = new ArrayList<ColumnElement>();
+				for (ColumnDefinitionView column : columnsDefined) {
+					if(column.isSelected()){
+//						column.setAsReadOnly(true);
+//						setBodyColumnVisible(flexTableTemplate, column.getColumnIndex(), false);
+						columnsSelected.add(column);
+					}
+				}
+				
+				if(columnsSelected.size()>0){
+					MultiFilterByColumns multi;
+					try {
+				
+						multi = new MultiFilterByColumns(columnsSelected);
+//						multi.getWindow().show();
+						LayoutContainer lc = multi.getPanel();
+						lc.setStyleAttribute("margin-bottom", "2px");
+						southContainer.add(lc);
+						southContainer.layout(true);
+					} catch (ColumnNotTypedException e) {
+						MessageBox.info("Set data type!", "You must set the data type for selected column/s before to add a filter", null).show();
+					}
+			
+				}else
+					MessageBox.info("No selection", "You must select (through checkbox) at least one column", null).show();
+				
+			}
+		});*/
+		
 
 		toolbar.add(addColumnButton);
 		toolbar.add(new SeparatorMenuItem());
 		toolbar.add(removeRowButton);
+		
+		toolbar.add(new FillToolItem());
+		
+//		toolbar.add(filterBySelection);
 		tableContainer.setTopComponent(toolbar);
 	}
 
-
+	public void enableFilter(boolean bool){
+//		filterBySelection.setEnabled(bool);
+		
+		if(!bool)
+			borderLayout.collapse(LayoutRegion.SOUTH);
+		else
+			borderLayout.expand(LayoutRegion.SOUTH);
+		
+		southContainer.setEnabled(bool);
+		
+	
+		/*
+		if(!bool)
+			borderLayout.collapse(LayoutRegion.SOUTH);
+		else
+			borderLayout.expand(LayoutRegion.SOUTH);
+			
+		borderLayout.layout();
+		*/
+//		southContainer.setVisible(bool);
+	}
+	
+	public void enanbleColumnTypes(boolean bool){
+//		flexTableContainer.setEnabled(bool);
+		
+		for (int i=0; i< columnsDefined.size(); i++) {
+			ColumnDefinitionView column = columnsDefined.get(i);
+			column.setColumnTypeAsReadOnly(!bool);
+			
+			if(bool)
+				setColumnTypeOpacity(i, 1);
+			else
+				setColumnTypeOpacity(i, 0.5);
+		}
+	}
+	
 	/**
 	 * Initialize this example.
 	 */
@@ -224,8 +344,6 @@ public class TemplatePanel {
 	public void inizializeTableTemplate() {
 		// Create a Flex Table
 		flexTableTemplate.addStyleName("FlexTableTemplate");
-//		initTableColumns(flexTableTemplate, 0, numColumns);
-		
 		flexTableTemplate.ensureDebugId("cwFlexTable");
 	}
 
@@ -253,15 +371,6 @@ public class TemplatePanel {
 
 			hp.add(columnDef.getSetColumnTypeViewManager().getTypingContainer());
 
-			CheckBox check = new CheckBox();
-			check.setStyleAttribute("margin-right", "10px");
-			
-			/*HorizontalPanel hp2 = new HorizontalPanel();
-			hp2.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
-			hp2.add(check);
-			hp2.add(columnDef.getColumnHeaderValue());
-			flexTable.setWidget(0, indexOffset, hp2);*/
-			
 			flexTable.setWidget(0, indexOffset, columnDef.getColumnHeaderValue());
 			flexTable.setWidget(1, indexOffset, hp);
 			
@@ -274,13 +383,6 @@ public class TemplatePanel {
 			rf.addStyleName(1, "FlexTableTemplate-other-rows");
 			rf.addStyleName(2, "FlexTableTemplate-other-rows");
 
-//			//TODO
-//			int size = columnsDefined.size();
-//			if(size>0 && indexOffset<size && columnsDefined.get(indexOffset)!=null)
-//				columnsDefined.set(indexOffset, columnDef);
-//			else
-//				columnsDefined.add(indexOffset, columnDef);
-			
 			try{
 				GWT.log("Setting indexOffset: "+indexOffset);
 				columnsDefined.set(indexOffset, columnDef);
@@ -301,6 +403,7 @@ public class TemplatePanel {
 			rf.addStyleName(rowIndex, "FlexTableTemplate-other-rows");
 		
 		flexTableTemplate.setWidget(rowIndex, columnIndex, widget);
+		
 	}
 	
 	public void clearCell(int rowIndex, int columnIndex){
@@ -318,17 +421,10 @@ public class TemplatePanel {
 	 * Add a row to the flex table.
 	 */
 	private void addColumn(FlexTable flexTable) {
-		
-//		int numRows = flexTable.getRowCount();
-//		int numColumns = flexTable.getCellCount(0);
 
 		initTableColumns(flexTable, numColumns, 1);
-		
 		setNumColumns(numColumns+1);
-		
 		validateTemplate();
-		
-//		flexTable.getFlexCellFormatter().setColSpan(0, 1, numRows + 1);
 	}
 	
 
@@ -403,25 +499,34 @@ public class TemplatePanel {
 	public List<ColumnDefinitionView> getColumnsDefined() {
 		return columnsDefined;
 	}
+	
 
-	/**
-	 * 
-	 */
 	public void validateTemplate() {
 		
 		boolean isValid = true;
 		for (ColumnDefinitionView col : columnsDefined) {
 			if(!col.isValid()){
 				isValid = false;
-				addNewSuggestion(TdTemplateConstants.SUGGESTION, TdTemplateConstants.PLEASE_SET_TYPE_TO_COLUMN_NUMBER+(col.getColumnIndex()+1));
+				refreshSuggestion(TdTemplateConstants.SUGGESTION, TdTemplateConstants.PLEASE_SET_TYPE_TO_COLUMN_NUMBER+(col.getColumnIndex()+1));
 				TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(false));
+//				enableFilter(false);
+//				resetFilterPanel();
 				break;
 			}
 		}
 		
 		if(isValid){
-			addNewSuggestion(TdTemplateConstants.TEMPLATE_COMPLETED, TdTemplateConstants.NOW_IS_POSSIBLE_TO_SAVE_THE_TEMPLATE_CREATED, TdTemplateAbstractResources.handsUP());
+			refreshSuggestion(TdTemplateConstants.TEMPLATE_COMPLETED, TdTemplateConstants.NOW_IS_POSSIBLE_TO_SAVE_THE_TEMPLATE_CREATED, TdTemplateAbstractResources.handsUP());
 			TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(true));
+			/*enableFilter(true);
+			enanbleColumnTypes(false);
+			BaloonPanel baloonPanel = new BaloonPanel("Do you want add filters?", true);
+			int zIndex = controller.getWindowZIndex();
+			int zi = zIndex+1;
+			baloonPanel.getElement().getStyle().setZIndex(zi);
+			baloonPanel.showRelativeTo(southContainer);
+			*/
+//			setCheckColumnsEnabled(true);
 		}
 	}
 }
